//
//  ApiKeysDataSource.swift
//  SampleApp
//
//  Created by Federico Zanetello on 6/27/19.
//  Copyright © 2019 David Zhang. All rights reserved.
//

final class ApiKeysDataSource {
    weak var dataSourceObserver: DataSourceListener?

    private var apiKeys: [String] {
        get {
            internalCheck()
            return UserDefaults.standard.apiKeys
        }
        set { UserDefaults.standard.apiKeys = newValue }
    }

    /// Makes sure that we always have at least one api key listed.
    ///
    /// The default api key is the one in the info.plist; It can be
    /// deleted, however it is restored after deleting all other
    /// keys.
    private func internalCheck() {
        guard UserDefaults.standard.apiKeys.isEmpty else { return }
        guard
            let apiKeyObject = Bundle.main.object(forInfoDictionaryKey: "EKOChatAPIKey"),
            let apiKey = apiKeyObject as? String else {
                assertionFailure("cannot get api key from info.plist")
                return
        }

        apiKeys = [apiKey]
    }

    func numberOfKeys() -> Int {
        return apiKeys.count
    }

    func key(for indexPath: IndexPath) -> String? {
        guard apiKeys.count > indexPath.row else { return nil }
        return apiKeys[indexPath.row]
    }

    func addKey(_ newKey: String) {
        guard !apiKeys.contains(newKey) else { return }

        apiKeys.append(newKey)
        dataSourceObserver?.dataSourceUpdated()
    }

    func removeKey(at index: Int) {
        guard apiKeys.count > index else { return }

        apiKeys.remove(at: index)
        dataSourceObserver?.dataSourceUpdated()
    }
}
