//
//  MembershipListDataSource.swift
//  SampleApp
//
//  Created by Federico Zanetello on 4/10/19.
//  Copyright © 2019 David Zhang. All rights reserved.
//

import EkoChat

final class MembershipListDataSource {
    private unowned var client: EkoClient
    private let channelId: String

    weak var dataSourceObserver: DataSourceListener?

    private var membershipsCollection: EkoCollection<EkoChannelMembership>?
    private var channelObject: EkoObject<EkoChannel>?

    private var membershipsToken: EkoNotificationToken?
    private var channelToken: EkoNotificationToken?

    init(client: EkoClient, channelId: String) {
        self.client = client
        self.channelId = channelId

        setupObserver()
    }

    private func setupObserver() {
        let channelRepository: EkoChannelRepository = EkoChannelRepository(client: client)
        channelObject = channelRepository.getChannel(channelId)

        guard let channel: EkoChannel = channelObject?.object else { return }
        membershipsCollection = channel.participation.memberships

        channelToken = channelObject?.observe { [weak self] _, _  in
            self?.dataSourceObserver?.dataSourceUpdated()
        }

        membershipsToken = membershipsCollection?.observe { [weak self] _, _, _ in
            self?.dataSourceObserver?.dataSourceUpdated()
        }
    }

    func numberOfMemberships() -> Int {
        return channelObject?.object?.memberCount ?? 0
    }

    func membership(for indexPath: IndexPath) -> EkoChannelMembership? {
        if
            let membershipsCollection = self.membershipsCollection,
            membershipsCollection.count() > indexPath.row {
            return membershipsCollection.object(at: UInt(indexPath.row))
        } else {
            membershipsCollection?.nextPage()
            return nil
        }
    }
}
