//
//  MembershipListTableViewController.swift
//  SampleApp
//
//  Created by Federico Zanetello on 4/10/19.
//  Copyright © 2019 David Zhang. All rights reserved.
//

import EkoChat
import UIKit

private let reuseIdentifier = "MemberTableViewCell"

final class MembershipListTableViewController: UITableViewController, DataSourceListener {
    weak var client: EkoClient!
    var channelId: String!

    private var dataSource: MembershipListDataSource!

    // MARK: Lifecycle

    override func viewDidLoad() {
        super.viewDidLoad()
        dataSource = MembershipListDataSource(client: client, channelId: channelId)
        dataSource.dataSourceObserver = self
        let nib = UINib(nibName: "SampleAppTableViewCell", bundle: nil)
        tableView.register(nib, forCellReuseIdentifier: reuseIdentifier)

        title = "Memberships"

        // remove empty rows
        tableView.tableFooterView = UIView()
    }

    // MARK: UITableViewDataSource

    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return dataSource.numberOfMemberships()
    }

    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        guard let membership = dataSource.membership(for: indexPath) else { return UITableViewCell() }
        let cell = tableView.dequeueReusableCell(withIdentifier: reuseIdentifier, for: indexPath)
        guard let sampleAppCell = cell as? SampleAppTableViewCell else { return UITableViewCell() }

        sampleAppCell.titleLabel?.text = membership.userId
        setTags(in: sampleAppCell, for: membership)
        return cell
    }

    private func setTags(in cell: SampleAppTableViewCell, for membership: EkoChannelMembership) {
        let tags: [(text: String, color: UIColor)] = self.tags(for: membership)

        let mutableAttributedString = NSMutableAttributedString()
        for tag in tags {
            let attributedTagString = NSAttributedString(string: " \(tag.text) ",
                attributes: [.backgroundColor: tag.color,
                             .foregroundColor: .white])
            mutableAttributedString.append(attributedTagString)
            mutableAttributedString.append(NSAttributedString(string: " "))
        }

        cell.subtitleLabel.attributedText = mutableAttributedString
    }

    private func tags(for membership: EkoChannelMembership) -> [(String, UIColor)] {
        let pastelColors: [UIColor] = [UIColor(named: "PastelRed"),
                                       UIColor(named: "PastelOrange"),
                                       UIColor(named: "PastelYellow"),
                                       UIColor(named: "PastelGreen"),
                                       UIColor(named: "PastelBlue"),
                                       UIColor(named: "PastelGreen"),
                                       UIColor(named: "NeonPink"),
                                       UIColor(named: "NeonRed"),
                                       UIColor(named: "NeonGreen"),
                                       UIColor(named: "NeonAzure")].compactMap { $0 }
        var tags: [(String, UIColor)] = []

        if membership.isBanned {
            tags.append(("Banned", pastelColors[tags.count % pastelColors.count]))
        }

        if membership.isMuted {
            tags.append(("Muted", pastelColors[tags.count % pastelColors.count]))
        }

        for role in membership.roles as? [String] ?? [] {
            tags.append((role, pastelColors[tags.count % pastelColors.count]))
        }

        return tags
    }

    // MARK: DataSourceListener

    func dataSourceUpdated() {
        tableView.reloadData()
    }
}
