//
//  EkoCustomViewController.swift
//  SampleApp
//
//  Created by Michael Abadi Santoso on 10/18/19.
//  Copyright © 2019 David Zhang. All rights reserved.
//

import UIKit

protocol EkoCustomViewControllerDelegate: AnyObject {
    func ekoCustom(_ viewController: EkoCustomViewController, willSendCustomDataWithData data: [String: Any])
    func ekoCustom(_ viewController: EkoCustomViewController, willUpdateCustomDataWithData data: [String: Any], onMessage message: EkoMessage?)
}

final class EkoCustomViewController: UIViewController {

    @IBOutlet private weak var keyField: UITextField!
    @IBOutlet private weak var valueField: UITextField!
    
    weak var delegate: EkoCustomViewControllerDelegate?
    
    private var message: EkoMessage?
    
    override func viewDidLoad() {
        super.viewDidLoad()
        if let message = message, let data = message.data?["data"] as? [String: Any] {
            keyField.text = data["key"] as? String
            valueField.text = data["value"] as? String
        }
    }
    
    func setMessage(message: EkoMessage) {
        self.message = message
    }
    
    @IBAction func handleSendButton(_ sender: Any) {
        guard let key = keyField.text, let value = valueField.text,
            !key.isEmpty, !value.isEmpty else { return }
        
        let map: [String: Any] = [
            "key": key,
            "value": value
        ]
        
        if let message = message {
            delegate?.ekoCustom(self, willUpdateCustomDataWithData: map, onMessage: message)
            navigationController?.dismiss(animated: true, completion: nil)
        } else {
            delegate?.ekoCustom(self, willSendCustomDataWithData: map)
            navigationController?.popViewController(animated: true)
        }
    }
    
    static func makeViewController() -> UIViewController {
        let sb = UIStoryboard(name: "Chats", bundle: nil)
        let vc = sb.instantiateViewController(withIdentifier: "EkoCustomViewController")
        return vc
    }
    
}
