//
//  EkoChatFileViewCell.swift
//  SampleApp
//
//  Created by Michael Abadi Santoso on 10/11/19.
//  Copyright © 2019 David Zhang. All rights reserved.
//

import Foundation

final class EkoChatFileViewCell: UITableViewCell, EkoChatTableViewCell {
    @IBOutlet weak var myImageView: UIImageView!
    @IBOutlet weak var myMessageLabelTrailingConstraint: NSLayoutConstraint!
    @IBOutlet weak var myDisplayNameLabelTrailingConstraint: NSLayoutConstraint!
    @IBOutlet weak var someoneMessageLabelLeadingingConstraint: NSLayoutConstraint!
    @IBOutlet weak var someoneDisplayNameLabelLeadingConstraint: NSLayoutConstraint!
    @IBOutlet weak var displayNameLabel: UILabel!
    @IBOutlet weak var timeLabel: UILabel!
    @IBOutlet weak var messageStatusImageView: UIImageView!
    
    private var messageId: String?
    
    var aspectRatioConstraint: NSLayoutConstraint? {
        didSet {
            if let oldValue = oldValue {
                oldValue.isActive = false
            }
            if let aspectRatioConstraint = aspectRatioConstraint {
                aspectRatioConstraint.isActive = true
            }
        }
    }
    var mediaRepo: EkoMediaRepository!
    
    private func set(image: UIImage) {
        myImageView.image = image
    }
    
    func display(_ message: EkoMessage, client: EkoClient) {
        messageId = message.messageId
        if message.userId == client.currentUserId {
            myMessageLabelTrailingConstraint.isActive = true
            myDisplayNameLabelTrailingConstraint.isActive = true
            someoneMessageLabelLeadingingConstraint.isActive = false
            someoneDisplayNameLabelLeadingConstraint.isActive = false
        } else {
            myMessageLabelTrailingConstraint.isActive = false
            myDisplayNameLabelTrailingConstraint.isActive = false
            someoneMessageLabelLeadingingConstraint.isActive = true
            someoneDisplayNameLabelLeadingConstraint.isActive = true
        }
        self.needsUpdateConstraints()
        self.setNeedsLayout()
        
        if message.isDeleted {
            displayNameLabel.text = "Message deleted"
        } else {
            let data = message.data as? [String: Any]
            let filename = data?["fileName"] as? String
            displayNameLabel.text = "\(filename ?? "File")"
            messageStatusImageView.image = symbol(for: message.syncState)
            myImageView.image = UIImage(named:"file")
            timeLabel.text = DateFormatter.localizedString(from: message.createdAtDate, dateStyle: .none, timeStyle: .short)
            if mediaRepo == nil || mediaRepo.client != client {
                mediaRepo = EkoMediaRepository(client: client)
            }
            if !message.isDeleted {
                self.aspectRatioConstraint = self.myImageView.widthAnchor.constraint(equalTo: self.myImageView.heightAnchor, multiplier: 1)
                self.myImageView.image = UIImage(named:"file")
                self.myImageView.tintColor = UIColor.gray
            }
        }
    }
    
    func downloadFile(completion: @escaping (URL) -> Void) {
        guard let messageId = messageId else { return }
        mediaRepo.getFileForMessageId(messageId) { [weak self] data, error in
            DispatchQueue.main.async {
                guard self?.messageId == messageId else { return }
                if
                    let data = data {
                    guard let documentsPath = NSSearchPathForDirectoriesInDomains(.documentDirectory, .userDomainMask, true).first else { return }
                    let mime = data.mimeType()
                    let filetypes = mime.components(separatedBy: "/")
                    let extention = filetypes.last ?? "pdf"
                    let filePath = "\(documentsPath)/\(messageId).\(extention)"
                    do {
                        try data.write(to: URL(fileURLWithPath: filePath))
                    } catch {
                        // failed to write file – bad permissions, bad filename, missing permissions, or more likely it can't be converted to the encoding
                    }
                    let url = URL(fileURLWithPath: filePath)
                    completion(url)
                }
            }
        }
    }
    
    private func symbol(for status: EkoSyncState) -> UIImage {
        switch status {
        case .default, .synced: return #imageLiteral(resourceName: "check")
        case .syncing: return #imageLiteral(resourceName: "check")
        case .error: return #imageLiteral(resourceName: "error")
        @unknown default: return UIImage()
        }
    }
    
    override func prepareForReuse() {
        super.prepareForReuse()
        aspectRatioConstraint = nil
        myImageView.image = nil
        timeLabel.text = ""
        messageStatusImageView.image = nil
    }
}
