//
//  ChannelListTableViewCellConfigurator.swift
//  SampleApp
//
//  Created by Federico Zanetello on 4/29/19.
//  Copyright © 2019 David Zhang. All rights reserved.
//

import Foundation
import EkoChat

final class ChannelListTableViewCellConfigurator {

    func configure(_ cell: SampleAppTableViewCell, with channel: EkoChannel) {
        setTitleLabel(for: cell, with: channel)
        setSubtitleLabel(for: cell, with: channel)
        setDetailLabel(for: cell, with: channel)
    }

    // MARK: Title

    private func setTitleLabel(for cell: SampleAppTableViewCell, with channel: EkoChannel) {
        cell.titleLabel.text = channel.displayName ?? channel.channelId
    }

    // MARK: Subtitle

    private func setSubtitleLabel(for cell: SampleAppTableViewCell, with channel: EkoChannel) {
        guard let channelTags = channel.tags as? [String] else { return }
        let tagsAttributedString: NSAttributedString = attributedString(for: channelTags)
        let channelEmoji: String = emoji(for: channel.channelType)
        let mutableAttributedString = NSMutableAttributedString(string: "\(channelEmoji) ")
        mutableAttributedString.append(tagsAttributedString)
        cell.subtitleLabel.attributedText = mutableAttributedString
    }

    private func attributedString(for tags: [String]) -> NSAttributedString {
        let colors: [UIColor] = ["PastelRed", "PastelOrange", "PastelYellow",
                                 "PastelGreen", "PastelBlue", "PastelPurple",
                                 "NeonPink", "NeonRed", "NeonGreen", "NeonAzure"].compactMap(UIColor.init)

        let mutableAttributedString = NSMutableAttributedString()
        for index in 0..<tags.count {
            let index = index % colors.count
            let attributedTagString = NSAttributedString(string: " \(tags[index]) ",
                attributes: [.backgroundColor: colors[index],
                             .foregroundColor: .white])
            mutableAttributedString.append(attributedTagString)
            mutableAttributedString.append(NSAttributedString(string: " "))
        }
        return mutableAttributedString
    }

    private func emoji(for channelType: EkoChannelType) -> String {
        switch channelType {
        case .standard:
            return "💬"
        case .private:
            return "🔒"
        case .byTypes:
            return "🗣"
        case .broadcast:
            return "🗣"
        case .conversation:
            return "💬"
        @unknown default:
            return "⁉️"
        }
    }

    // MARK: Detail

    private func setDetailLabel(for cell: SampleAppTableViewCell, with channel: EkoChannel) {
        let detailText: String
        if
            channel.currentUserMembership == .member,
            channel.unreadCount != 0 {
            detailText = "🆕 \(channel.unreadCount)"
        } else {
            detailText = channel.currentUserMembership.description
        }
        cell.detailLabel.text = detailText
    }
}

extension EkoChannelMembershipType: CustomStringConvertible {
    public var description: String {
        switch self {
        case .banned: return "banned"
        case .member: return "member"
        case .none: return "notMember"
        @unknown default: return "unknown"
        }
    }
}
