//
//  FeedModels.swift
//  SampleApp
//
//  Created by Nishan Niraula on 4/28/20.
//  Copyright © 2020 David Zhang. All rights reserved.
//

import Foundation

enum FeedCellItem {
    case header
    case footer
    case content(type: FeedItemType)
    case reaction
    
    var height: CGFloat {
        switch self {
        case .header:
            return 70
        case .content(let type):
            switch type {
            case .text, .image:
                return UITableView.automaticDimension
            }
        case .reaction:
            return 40
        case .footer:
            return 60
        }
    }
}

enum FeedItemType {
    case text
    case image
}

protocol FeedItemModel {
    var type: FeedItemType { get }
    var userName: String { get set }
    var date: String { get set }
}

// Sample models. Will change later when integrating with sdk
struct TextFeedModel: FeedItemModel {
    var type: FeedItemType = .text
    var text: String
    var userName: String
    var date: String
    
    init(text: String, userName: String, date: String) {
        self.text = text
        self.userName = userName
        self.date = date
    }
}

struct ImageFeedModel: FeedItemModel {
    var type: FeedItemType = .image
    var image: UIImage
    var userName: String
    var date: String
    
    init(image: UIImage, userName: String, date: String) {
        self.image = image
        self.userName = userName
        self.date = date
    }
}

protocol FeedItemAction {
    var title: String { get }
    var id: String { get }
}

enum FeedItemDefaultAction: FeedItemAction {
    case edit
    case delete
    case like
    case comment
    case sortLastCreated
    case sortFirstCreated
    
    var title: String {
        switch self {
        case .edit:
            return "Edit"
        case .delete:
            return "Delete"
        case .like:
            return "Like"
        case .comment:
            return "Comment"
        case .sortLastCreated:
            return "Last Created"
        case .sortFirstCreated:
            return "First Created"
        }
    }
    
    var id: String {
        switch self {
        case .edit:
            return "feed.edit"
        case .delete:
            return "feed.delete"
        case .like:
            return "feed.like"
        case .comment:
            return "feed.comment"
        case .sortLastCreated:
            return "feed.sort.last.created"
        case .sortFirstCreated:
            return "feed.sort.first.created"
        }
    }
}
