//
//  NewUserPostsViewController.swift
//  SampleApp
//
//  Created by Nishan Niraula on 4/27/20.
//  Copyright © 2020 David Zhang. All rights reserved.
//

import UIKit

/*
 * Note:
 *
 * This class displays the screen to edit/create new post.
 */
class NewUserPostsViewController: UIViewController {
    
    @IBOutlet weak var textView: UITextView!
    
    var feedManager: UserPostsFeedManager!
    
    enum Mode {
        case create
        case edit
    }
    
    var currentMode: Mode = .create
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        setupViews()
    }
    
    func setupViews() {
        textView.keyboardDismissMode = .onDrag
        textView.font = UIFont.systemFont(ofSize: 16, weight: .regular)
        
        updateViewsForCurrentMode()
    }
    
    func updateViewsForCurrentMode() {
        
        let postButton: UIBarButtonItem
        let cancelButton: UIBarButtonItem = UIBarButtonItem(title: "Cancel", style: .plain, target: self, action: #selector(onCancelButtonTap))
        
        
        switch currentMode {
        case .create:
            self.title = "Create Post"
            
            postButton = UIBarButtonItem(title: "Post", style: .plain, target: self, action: #selector(onPostButtonTap))
        case .edit:
            self.title = "Edit Post"
            
            postButton = UIBarButtonItem(title: "Update", style: .plain, target: self, action: #selector(onPostButtonTap))
            
            // Show that edited post
            let post = feedManager.getEditPostData()
            textView.text = post.text
        }
        
        self.navigationItem.rightBarButtonItem = postButton
        self.navigationItem.leftBarButtonItem = cancelButton
    }
    
    @objc func onPostButtonTap() {
        let postText = textView.text ?? ""
        guard !postText.isEmpty else { return }
        
        switch currentMode {
        case .create:
            feedManager.createPost(text: postText) { [weak self] isSuccess in
                self?.showAlertAndDismiss(isSuccess: isSuccess)
            }
        case .edit:
            feedManager.updatePost(text: postText) { [weak self] (isSuccess) in
                self?.showAlertAndDismiss(isSuccess: isSuccess)
            }
        }
    }
    
    func showAlertAndDismiss(isSuccess: Bool) {
        
        var alertMessage = isSuccess ? "Post successfully created" : "Error while creating post"
        if currentMode == .edit {
            alertMessage = isSuccess ? "Post successfully updated" : "Error while updating post"
        }
        
        let alert = UIAlertController(title: "", message: alertMessage, preferredStyle: .alert)
        let action = UIAlertAction(title: "Dismiss", style: .cancel, handler: { action in
            self.dismiss(animated: true, completion: nil)
        })
        alert.addAction(action)
        
        self.present(alert, animated: true, completion: nil)
    }
    
    @objc func onCancelButtonTap() {
        self.dismiss(animated: true, completion: nil)
    }
}
