//
//  MoreTableViewController.swift
//  SampleApp
//
//  Created by Federico Zanetello on 6/6/19.
//  Copyright © 2019 David Zhang. All rights reserved.
//

import Foundation

protocol MoreTableViewControllerDelegate: AnyObject {
    func moreTable(_ viewController: MoreTableViewController, willChangeChannelType channelType: EkoChannelType)
}

final class MoreTableViewController: UITableViewController {
    /// To be injected.
    weak var client: EkoClient!
    weak var delegate: MoreTableViewControllerDelegate?

    private var channelRepository: EkoChannelRepository?
    
    private var createToken: EkoNotificationToken?
    
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        super.prepare(for: segue, sender: sender)
        switch segue.destination {
        case let userPushNotificationsTableViewController as UserPushNotificationsTableViewController:
            userPushNotificationsTableViewController.client = client
        case let pushNotificationsViewController as PushNotificationsTableViewController:
            let userPushNotificationManager = UserLevelPushNotificationManager(client: client)
            pushNotificationsViewController.pushNotificationManager = userPushNotificationManager
        case let userViewController as UserScreenViewController:
            userViewController.client = client
        default:
            break
        }
    }
    
    override func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        switch indexPath.section {
        case 2:
            updateUserMeta()
        case 3:
            switch indexPath.row {
            case 0:
                changeChannelType(.standard)
            case 1:
                changeChannelType(.private)
            case 2:
                changeChannelType(.byTypes)
            case 3:
                changeChannelType(.broadcast)
            case 4:
                changeChannelType(.conversation)
            default:
                break
            }
        case 4:
            switch indexPath.row {
            case 0:
                createChannel(type: .standard)
            case 1:
                createChannel(type: .private)
            default:
                break
            }
        default:
            break
        }
    }
    
    private func createChannel(type: EkoChannelCreateType) {
        channelRepository = EkoChannelRepository(client: client)
        let alertController = UIAlertController(title: "Create New Channel",
                                                message: "Add Display Namme",
                                                preferredStyle: .alert)
        alertController.addTextField { textField in
            textField.placeholder = "Type the display name here"
        }
        let newChannelId = String.randomString(length: 10)
        let addAction = UIAlertAction(title: "Create",
                                      style: .default) { [weak self] _ in
                                        guard
                                            let name = alertController
                                                .textFields?
                                                .first?
                                                .text,
                                            !name.isEmpty else { return }
                                        self?.createToken = self?.channelRepository?.createChannel(newChannelId, displayName: name, type: type, metadata: [:], tags: [], users: []).observeOnce({ object, error in
                                            self?.createToken?.invalidate()
                                            guard object.dataStatus == .fresh else {
                                                return
                                            }
                                            let alertController = UIAlertController(title: "Create Channel Success", message: nil, preferredStyle: .alert)
                                            alertController.addAction(UIAlertAction(title: "OK", style: .default, handler: nil))
                                            self?.present(alertController, animated: true, completion: nil)
                                        })
        }
        alertController.addAction(addAction)
        let cancelAction = UIAlertAction(title: "Cancel", style: .cancel, handler: nil)
        alertController.addAction(cancelAction)
        present(alertController, animated: true, completion: nil)
    }
    
    private func changeChannelType(_ type: EkoChannelType) {
        delegate?.moreTable(self, willChangeChannelType: type)
        navigationController?.popViewController(animated: true)
    }
    
    private func updateUserMeta() {
        let alertController = UIAlertController(title: "Update User Meta",
                                                message: "",
                                                preferredStyle: .alert)
        alertController.addTextField { textField in
            textField.placeholder = "Type the meta here"
        }
        let addAction = UIAlertAction(title: "Update",
                                      style: .default) { [weak self] _ in
                                        guard
                                            let meta = alertController
                                                .textFields?
                                                .first?
                                                .text,
                                            !meta.isEmpty else { return }
                                        let data = [
                                            "meta": meta
                                        ]
                                        self?.client.setUserMetadata(data, completion: nil)
        }
        alertController.addAction(addAction)
        let cancelAction = UIAlertAction(title: "Cancel", style: .cancel, handler: nil)
        alertController.addAction(cancelAction)
        present(alertController, animated: true, completion: nil)
    }
}
