//
//  UserDatasource.swift
//  SampleApp
//
//  Created by Michael Abadi Santoso on 4/17/20.
//  Copyright © 2020 David Zhang. All rights reserved.
//

import UIKit

final class UserDatasource {
    private var userCollection: EkoCollection<EkoUser>
    private var usersToken: EkoNotificationToken?
    weak var dataSourceObserver: DataSourceListener?

    init(userCollection: EkoCollection<EkoUser>) {
        self.userCollection = userCollection
        updateToken()
    }
    
    private func updateToken() {
        usersToken?.invalidate()
        usersToken = userCollection.observe { [weak self] collection, _, _ in
            self?.dataSourceObserver?.dataSourceUpdated()
        }
    }
    
    func setCollection(userCollection: EkoCollection<EkoUser>) {
        self.userCollection = userCollection
        updateToken()
    }

    func numberOfUsers() -> Int {
        return Int(userCollection.count())
    }

    func user(for indexPath: IndexPath) -> EkoUser? {
        return userCollection.object(at: UInt(indexPath.row))
    }

    func loadMore() {
        userCollection.nextPage()
    }
    
    func hasNext() -> Bool {
        return userCollection.hasNext
    }
}
