//
//  UserScreenViewController.swift
//  SampleApp
//
//  Created by Michael Abadi Santoso on 4/17/20.
//  Copyright © 2020 David Zhang. All rights reserved.
//

import UIKit

final class UserScreenViewController: UIViewController {

    @IBOutlet private weak var textField: UITextField!
    
    @IBOutlet private weak var tableView: UITableView!

    /// To be injected.
    weak var client: EkoClient!
    
    private var userRepository: EkoUserRepository?
    private var datasource: UserDatasource?
    
    override func viewDidLoad() {
        super.viewDidLoad()
        tableView.delegate = self
        tableView.dataSource = self
        tableView.register(UINib(nibName: "UserViewCell", bundle: nil), forCellReuseIdentifier: "UserViewCell")
        userRepository = EkoUserRepository(client: client)
        let collection = userRepository?.getAllUsersSorted(by: .displayName)
        datasource = UserDatasource(userCollection: collection!)
        datasource?.dataSourceObserver = self
        dataSourceUpdated()
    }
    
    @IBAction func handleSearchButton(_ sender: Any) {
        datasource?.setCollection(userCollection: userRepository!.searchUser(textField.text ?? "", sortBy: .displayName))
        dataSourceUpdated()
    }
    
    @IBAction func handleFirstCreated(_ sender: Any) {
        if let text = textField.text, !text.trimmingCharacters(in: .whitespacesAndNewlines).isEmpty {
            datasource?.setCollection(userCollection: userRepository!.searchUser(text, sortBy: .firstCreated))
        } else {
            datasource?.setCollection(userCollection: userRepository!.getAllUsersSorted(by: .firstCreated))
        }
        dataSourceUpdated()
    }
        
    @IBAction func handleLastCreated(_ sender: Any) {
        if let text = textField.text, !text.trimmingCharacters(in: .whitespacesAndNewlines).isEmpty {
            datasource?.setCollection(userCollection: userRepository!.searchUser(text, sortBy: .lastCreated))
        } else {
            datasource?.setCollection(userCollection: userRepository!.getAllUsersSorted(by: .lastCreated))
        }
        dataSourceUpdated()
    }
    
}

extension UserScreenViewController: UITableViewDataSource {
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return datasource?.numberOfUsers() ?? 0
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        return tableView.dequeueReusableCell(withIdentifier: "UserViewCell", for: indexPath)
    }    
}

extension UserScreenViewController: UITableViewDelegate {
    func tableView(_ tableView: UITableView, willDisplay cell: UITableViewCell, forRowAt indexPath: IndexPath) {
        guard let datasource = datasource else { return }
        let cell = cell as? UserViewCell
        let name: String
        if let displayName = datasource.user(for: indexPath)?.displayName {
            name = "Display Name : \(displayName)"
        } else if let userId = datasource.user(for: indexPath)?.userId {
            name = "No display name detected. User ID : \(userId)"
        } else {
            name = ""
        }
        cell?.setTitle(name: name)
        if indexPath.row == datasource.numberOfUsers() - 1 {
            if datasource.hasNext() {
                datasource.loadMore()
            }
        }
    }
}

extension UserScreenViewController: DataSourceListener {

    func dataSourceUpdated() {
        tableView.reloadData()
    }

}
